open VecOps
open GobaUtils

let layout = [|[|0; 0; 1; 1; 0; 0|];
	       [|0; 0; 1; 1; 0; 0|];
	       [|1; 1; 0; 0; 1; 1|];
	       [|1; 1; 0; 0; 1; 1|];
	       [|0; 0; 1; 1; 0; 0|];
	       [|0; 0; 1; 1; 0; 0|]|]
let layout_size = 6, 6
let tile = 11, 18

let base_attack_sound = lazy (Sdlmixer.loadWAV "base-attack.wav")

let recharge_sound = lazy (Sdlmixer.loadWAV "recharge.wav")

class t id init constr =
  let bitmap = lazy (Bitmaps.bitmap (`Block (11, 18, constr.Constr.base_color))) in
  let bitmap_offset = Vector.int2 ((-2.5, -2.5) *|. Vector.float2 tile) in
  let cmask = 
    lazy 
      (let module C = CollisionMask in
       let module B = Bitmaps in
       let m = C.create_mask (fst tile * 6) (snd tile * 6) in
       let block_mask = B.collision_mask_of_surface (snd (Lazy.force bitmap)) in
	 ( for xc = 0 to fst layout_size - 1 do
	     ( for yc = 0 to snd layout_size - 1 do
		 if layout.(yc).(xc) = 1 then
		   C.blit m (((xc, yc) *| tile), block_mask)
	       done )
	   done );
	 m
      ) in

object (self : 'a)
  inherit Object.t id
  inherit Collision.t

  val mutable at = Object.default init (0.0, 0.0)
  val mutable hp = 500
  val mutable next_recharge = 0.0

  method set_location at' = at <- at'

  method location = at
  method set_speed speed' = ()
  method speed = 0.0, 0.0
  method render (gfx:#Gfx.t) (_:Object.render_flags) at = 
    let ofs, surface = Lazy.force bitmap in
    let ofs = ofs +| bitmap_offset in
      ( for xc = 0 to fst layout_size - 1 do
	    ( for yc = 0 to snd layout_size - 1 do
		  if layout.(yc).(xc) = 1 then
		    gfx#blit (at +| ofs +| (xc, yc) *| tile) surface
	      done )
	done )

  method do_recharge world =
    if self#self_owned then
      let at0 = at +|. Vector.float2 bitmap_offset in
      let size = (6.0, 6.0) *|. Vector.float2 tile in
      let objs = 
	world.Object.World.world_objs 
	  (at0, at0 +|. size)
	  (fun _ -> true)
      in
	if List.fold_left (fun recharged o -> o#recharge || recharged) false objs then
	  if not (Sdlmixer.playing_channel 7) then
	    Sdlmixer.play_channel ~channel:7 (Lazy.force recharge_sound)
	    
  method tick delta world = 
    next_recharge <- next_recharge -. delta;
    if next_recharge <= 0.0 then
      begin
	next_recharge <- 0.2;
	self#do_recharge world;
      end

  method move f = 
    at <- f at

  method is_alive = hp > 0

  method collision_area = 
    let at0 = at +|. Vector.float2 bitmap_offset in
    let size = (6.0, 6.0) *|. Vector.float2 tile in
      (*Printf.printf "Collision area: %f,%f + %f,%f\n%!" (fst at0) (snd at0) (fst size) (snd size);*)
      (at0, at0 +|. size), Lazy.force cmask

  method get_hp = hp

  method cause_damage damage =
    if not (Sdlmixer.playing_channel 7) then
      Sdlmixer.play_channel ~loops:~-1 ~channel:7 (Lazy.force base_attack_sound);
    Sdlmixer.expire_channel 7 (Some 2.0);
    hp <- max 0 (hp - damage)

  initializer
    (match init with 
       | Object.Init _ -> ()
       | Object.State (state0, age) -> self#set_state state0 age);
    self#change;
end
