open VecOps

let damage = 40

let radius = 40

let bitmap_nonactive = 
  lazy (let (ofs, bmap) = Bitmaps.bitmap (`Block (5, 5, (255, 0, 0))) in
	  (ofs -| (2, 2), bmap))

(*let bitmap_active = 
  lazy (let (ofs, bmap) = Bitmaps.bitmap (`Block (5, 5, (128, 128, 128))) in
	  (ofs -| (2, 2), bmap))*)

let cmask = lazy (fst (Lazy.force bitmap_nonactive), Bitmaps.collision_mask_of_surface (snd (Lazy.force bitmap_nonactive)))

(*let cofs, cmask = CollisionMask.circle_loc_mask radius*)

let explosion_sound = lazy (Sdlmixer.loadWAV "explosion.wav")
let activation_sound = lazy (Sdlmixer.loadWAV "mine-activated.wav")
let triggered_sound = lazy (Sdlmixer.loadWAV "mine-triggered.wav")

let flash_time = 0.2

class t id init constr =
  let flash_cycle = sqrt (Random.float 20.0) +. 5.0 in
object (self)
  inherit Object.t id
    
  val mutable at = Object.default init (0.0, 0.0)
  val mutable alive = true
  val mutable trig_time = 1.0 
  val mutable activated = false
  val mutable explosion_time = None
  val mutable flash_counter = Random.float flash_cycle

  method location = at
  method speed = (0.0, 0.0)

  method set_speed _ = ()
  method set_location at' = at <- at'

  method is_alive = alive

  method render gfx _ at =
    if (self#self_owned && not activated) || 
      explosion_time <> None ||
      flash_counter < flash_time then
      let bitmap = bitmap_nonactive in
      let ofs, surface = Lazy.force bitmap in
	gfx#blit (at +| ofs) surface

  method tick delta world =
    if alive then
      begin
	flash_counter <- mod_float (flash_counter +. delta) flash_cycle;
	if not activated then
	  begin
	    trig_time <- trig_time -. delta;
	    activated <- trig_time <= 0.0;
	    if activated then
	      Sdlmixer.play_channel ~channel:8 (Lazy.force activation_sound);
	  end
	else
	  begin
	    match explosion_time with
	      | None -> ()
	      | Some t when t < 0.0 ->
		  self#detonate world
	      | Some t -> 
		  explosion_time <- Some (t -. delta)
	  end
      end

  method destructible = false

  method collision_area =
    let offset = Vector.float2 (fst (Lazy.force cmask)) in
      ((at +|. offset, 
	at +|. offset +|. Vector.float2 (CollisionMask.size (snd (Lazy.force cmask)))), 
       snd (Lazy.force cmask))
	
  method trigger =
    Sdlmixer.play_channel ~channel:8 (Lazy.force triggered_sound);
    explosion_time <- Some 0.1

  method detonate world =
    Sdlmixer.play_channel ~channel:8 (Lazy.force explosion_sound);
    self#change;
    self#set_self_owned true;
    alive <- false;
    let objs = 
      world.Object.World.world_objs 
	(at -|. Vector.dup (float radius *. 2.0),
	 at +|. Vector.dup (float radius *. 2.0))
	(fun obj -> Vector.abs2 (obj#location -|. at) <= float radius)
    in
      List.iter (fun o -> 
		   let distance = (Vector.abs2 (o#location -|. at) /. float radius) +. 1.0 in
		     o#impact world (int_of_float (float damage /. distance))) objs;
      world.Object.World.world_explode at (0.0, 0.0)

  method cause_damage damage =
    alive <- false

  method impact world _ =
    if alive then
      self#trigger;

  method collide world =
    if alive && activated && explosion_time = None then
      begin
	self#trigger;
	true
      end
    else
      false

  initializer
    (match init with 
       | Object.Init _ -> ()
       | Object.State (state0, age) -> self#set_state state0 age);
    self#change;
end
